@@ -1181,7 +1181,7 @@ void CControlSocket::Sleep(fz::duration const& delay)
 bool CControlSocket::InitBufferPool(bool use_shm)
 {
 	if (!buffer_pool_) {
-		buffer_pool_.emplace(logger_, 8, 0, use_shm);
+		buffer_pool_.emplace(engine_.GetThreadPool(), logger_, 8, 0, use_shm);
 	}
 	return *buffer_pool_;
 }
