@@ -1,4 +1,4 @@
-#include <filezilla.h>
+#include "filezilla.h"
 #include "directorycache.h"
 
 #include <assert.h>
@@ -273,7 +273,7 @@ bool CDirectoryCache::InvalidateFile(CServer const& server, CServerPath const& p
 			}
 		}
 		else {
-			if (path.CmpNoCase(entry.listing.path)) {
+			if (!path.equal_nocase(entry.listing.path)) {
 				continue;
 			}
 		}
@@ -328,7 +328,7 @@ bool CDirectoryCache::UpdateFile(CServer const& server, CServerPath const& path,
 
 	for (tCacheIter iter = sit->cacheList.begin(); iter != sit->cacheList.end(); ++iter) {
 		auto & entry = const_cast<CCacheEntry&>(*iter);
-		if (path.CmpNoCase(entry.listing.path)) {
+		if (!path.equal_nocase(entry.listing.path)) {
 			continue;
 		}
 
@@ -408,7 +408,7 @@ bool CDirectoryCache::RemoveFile(CServer const& server, CServerPath const& path,
 
 	for (tCacheIter iter = sit->cacheList.begin(); iter != sit->cacheList.end(); ++iter) {
 		auto & entry = const_cast<CCacheEntry&>(*iter);
-		if (path.CmpNoCase(entry.listing.path)) {
+		if (!path.equal_nocase(entry.listing.path)) {
 			continue;
 		}
 
@@ -589,6 +589,40 @@ void CDirectoryCache::Rename(CServer const& server, CServerPath const& pathFrom,
 	InvalidateServer(server);
 }
 
+void CDirectoryCache::UpdateOwnerGroup(CServer const& server, CServerPath const& path, std::wstring const& filename, std::wstring& ownerGroup)
+{
+	fz::scoped_lock lock(mutex_);
+
+	tServerIter sit = GetServerEntry(server);
+	if (sit == m_serverList.end()) {
+		return;
+	}
+
+	tCacheIter iter;
+	bool is_outdated = false;
+	bool found = Lookup(iter, sit, path, true, is_outdated);
+	if (found) {
+		auto & listing = const_cast<CDirectoryListing&>(iter->listing);
+		size_t i;
+		for (i = 0; i < listing.size(); ++i) {
+			if (listing[i].name == filename) {
+				break;
+			}
+		}
+		if (i != listing.size()) {
+			if (!listing[i].is_dir()) {
+				listing.get(i).ownerGroup.get() = ownerGroup;
+				listing.ClearFindMap();
+			}
+			return;
+		}
+	}
+
+	// We know nothing, be on the safe side and invalidate everything.
+	InvalidateServer(server);
+}
+
+
 CDirectoryCache::tServerIter CDirectoryCache::CreateServerEntry(CServer const& server)
 {
 	for (tServerIter iter = m_serverList.begin(); iter != m_serverList.end(); ++iter) {
@@ -649,6 +683,8 @@ void CDirectoryCache::Prune()
 
 void CDirectoryCache::SetTtl(fz::duration const& ttl)
 {
+	fz::scoped_lock lock(mutex_);
+
 	if (ttl < fz::duration::from_seconds(30)) {
 		ttl_ = fz::duration::from_seconds(30);
 	}
