@@ -1,18 +1,22 @@
-#include <filezilla.h>
+#include "../filezilla.h"
 
 #include "chmod.h"
 #include "../directorycache.h"
 
+namespace {
 enum chmodStates
 {
 	chmod_init,
 	chmod_waitcwd,
 	chmod_chmod
 };
+}
 
 int CFtpChmodOpData::Send()
 {
 	if (opState == chmod_init) {
+		log(logmsg::status, _("Setting permissions of '%s' to '%s'"), command_.GetPath().FormatFilename(command_.GetFile()), command_.GetPermission());
+
 		controlSocket_.ChangeDir(command_.GetPath());
 		opState = chmod_waitcwd;
 		return FZ_REPLY_CONTINUE;
