@@ -40,6 +40,9 @@ static const t_protocolInfo protocolInfos[] = {
 	{ INSECURE_WEBDAV, L"webdav",    true,  true,    80, true,  fztranslate_mark("WebDAV using HTTP (insecure)"),                          L"http" },
 	{ RACKSPACE,       L"rackspace", true,  true,   443, false, "Rackspace Cloud Storage",                                                 L"https" },
 	{ STORJ_GRANT,     L"storj",     true,  true,  7777, true,  fztranslate_mark("Storj - Decentralized Cloud Storage"),                   L"" },
+	{ S3_SSO,          L"s3sso",     true,  true,   443, false, "S3 via IAM Identity Center (formerly SSO)",                               L"" },
+	{ GOOGLE_CLOUD_SVC_ACC,  L"googleacc", true,  true,   443, false, fztranslate_mark("Google Cloud Storage with Service Account"),       L"https" },
+	{ CLOUDFLARE_R2,   L"r2",        true,  true,   443, false, "Cloudflare R2",                                                           L"https" },
 	{ UNKNOWN,         L"",          false, false,   21, false, "",                                                                        L"" }
 };
 
@@ -399,7 +402,7 @@ std::wstring CServer::Format(ServerFormat formatType, Credentials const& credent
 				}
 			}
 			else {
-				server = fz::percent_encode_w(user) + L"@" + server;
+				server = user + L"@" + server;
 			}
 		}
 	}
@@ -587,7 +590,8 @@ bool CServer::ProtocolHasFeature(ServerProtocol const protocol, ProtocolFeature
 			protocol == SFTP ||
 			protocol == AZURE_BLOB || protocol == AZURE_FILE || protocol == B2 || protocol == BOX ||
 			protocol == DROPBOX || protocol == GOOGLE_CLOUD || protocol == GOOGLE_DRIVE ||
-			protocol == ONEDRIVE || protocol == S3 || protocol == SWIFT || protocol == WEBDAV) {
+			protocol == ONEDRIVE || protocol == S3 || protocol == S3_SSO || protocol == CLOUDFLARE_R2 ||
+			protocol == SWIFT || protocol == WEBDAV || protocol == GOOGLE_CLOUD_SVC_ACC) {
 			return true;
 		}
 		break;
@@ -604,7 +608,7 @@ bool CServer::ProtocolHasFeature(ServerProtocol const protocol, ProtocolFeature
 		}
 		break;
 	case ProtocolFeature::RecursiveDelete:
-		if (protocol == GOOGLE_DRIVE || protocol == DROPBOX || protocol == ONEDRIVE || protocol == B2) {
+		if (protocol == BOX || protocol == GOOGLE_DRIVE || protocol == DROPBOX || protocol == ONEDRIVE || protocol == B2) {
 			return true;
 		}
 		break;
@@ -615,7 +619,8 @@ bool CServer::ProtocolHasFeature(ServerProtocol const protocol, ProtocolFeature
 		}
 		break;
 	case ProtocolFeature::TemporaryUrl:
-		if (protocol == B2 || protocol == S3 || protocol == DROPBOX || protocol == AZURE_BLOB || protocol == AZURE_FILE) {
+		if (protocol == B2 || protocol == S3 || protocol == S3_SSO || protocol == DROPBOX ||
+			protocol == AZURE_BLOB || protocol == AZURE_FILE) {
 			return true;
 		}
 		break;
@@ -637,20 +642,30 @@ bool CServer::ProtocolHasFeature(ServerProtocol const protocol, ProtocolFeature
 				return true;
 		}
 		break;
+	case ProtocolFeature::PruneOldVersions:
+		if (protocol == B2 || protocol == BOX || protocol == GOOGLE_DRIVE || protocol == S3 || protocol == S3_SSO) {
+			return true;
+		}
+		break;
 	case ProtocolFeature::ListVersions:
 		if (protocol == B2 || protocol == BOX || protocol == DROPBOX ||
-			protocol == ONEDRIVE || protocol == GOOGLE_DRIVE || protocol == S3) {
+			protocol == ONEDRIVE || protocol == GOOGLE_DRIVE || protocol == S3 || protocol == S3_SSO) {
 			return true;
 		}
 		break;
 	case ProtocolFeature::DownloadVersion:
 		if (protocol == B2 || protocol == BOX || protocol == DROPBOX ||
-			protocol == GOOGLE_DRIVE || protocol == S3) {
+			protocol == GOOGLE_DRIVE || protocol == S3 || protocol == S3_SSO) {
 			return true;
 		}
 		break;
 	case ProtocolFeature::DeleteVersion:
-		if (protocol == B2 || protocol == BOX || protocol == GOOGLE_DRIVE || protocol == S3) {
+		if (protocol == B2 || protocol == BOX || protocol == GOOGLE_DRIVE || protocol == S3 || protocol == S3_SSO) {
+			return true;
+		}
+		break;
+	case ProtocolFeature::Share:
+		if (protocol == BOX || protocol == DROPBOX || protocol == GOOGLE_DRIVE || protocol == ONEDRIVE) {
 			return true;
 		}
 		break;
@@ -757,6 +772,9 @@ LogonType GetLogonTypeFromName(std::wstring const& name)
 	else if (name == _("Profile")) {
 		return LogonType::profile;
 	}
+	else if (name == _("Application Default Credentials")) {
+		return LogonType::adc;
+	}
 	else {
 		return LogonType::anonymous;
 	}
@@ -780,6 +798,8 @@ std::wstring GetNameFromLogonType(LogonType type)
 		return _("Account");
 	case LogonType::profile:
 		return _("Profile");
+	case LogonType::adc:
+		return _("Application Default Credentials");
 	default:
 		return _("Anonymous");
 	}
@@ -875,6 +895,8 @@ std::vector<LogonType> GetSupportedLogonTypes(ServerProtocol protocol)
 		return {LogonType::anonymous, LogonType::normal, LogonType::ask, LogonType::interactive, LogonType::key};
 	case S3:
 		return {LogonType::anonymous, LogonType::normal, LogonType::ask, LogonType::profile};
+	case S3_SSO:
+		return {LogonType::interactive, LogonType::profile};
 	case STORJ:
 	case STORJ_GRANT:
 	case AZURE_FILE:
@@ -895,6 +917,10 @@ std::vector<LogonType> GetSupportedLogonTypes(ServerProtocol protocol)
 	case HTTPS:
 	case UNKNOWN:
 		return {LogonType::anonymous};
+	case GOOGLE_CLOUD_SVC_ACC:
+		return {LogonType::key, LogonType::adc};
+	case CLOUDFLARE_R2:
+		return {LogonType::normal, LogonType::interactive};
 	}
 
 	return {LogonType::anonymous};
@@ -934,6 +960,15 @@ std::vector<ParameterTraits> const& ExtraServerParameterTraits(ServerProtocol pr
 			}();
 			return ret;
 		}
+	case GOOGLE_CLOUD_SVC_ACC:
+		{
+			static std::vector<ParameterTraits> const ret = []() {
+				std::vector<ParameterTraits> ret;
+				ret.emplace_back(ParameterTraits{"oauth_identity", ParameterSection::extra, ParameterTraits::optional | ParameterTraits::custom, std::wstring(), std::wstring()});
+				return ret;
+			}();
+			return ret;
+		}
 	case SWIFT:
 		{
 			static std::vector<ParameterTraits> const ret = []() {
@@ -957,6 +992,7 @@ std::vector<ParameterTraits> const& ExtraServerParameterTraits(ServerProtocol pr
 			return ret;
 		}
 	case S3:
+	case S3_SSO:
 		{
 			static std::vector<ParameterTraits> const ret = []() {
 				std::vector<ParameterTraits> ret;
@@ -967,58 +1003,74 @@ std::vector<ParameterTraits> const& ExtraServerParameterTraits(ServerProtocol pr
 				ret.emplace_back(ParameterTraits{"stsmfaserial", ParameterSection::extra, ParameterTraits::optional | ParameterTraits::custom, std::wstring(), std::wstring()});
 				ret.emplace_back(ParameterTraits{"region", ParameterSection::extra, ParameterTraits::optional | ParameterTraits::custom, std::wstring(), std::wstring()});
 				ret.emplace_back(ParameterTraits{"original_profile", ParameterSection::extra, ParameterTraits::optional | ParameterTraits::custom, std::wstring(), std::wstring()});
+				ret.emplace_back(ParameterTraits{"ssoregion", ParameterSection::extra, ParameterTraits::optional | ParameterTraits::custom, std::wstring(), std::wstring()});
+				ret.emplace_back(ParameterTraits{"ssorole", ParameterSection::extra, ParameterTraits::optional | ParameterTraits::custom, std::wstring(), std::wstring()});
+				ret.emplace_back(ParameterTraits{"ssourl", ParameterSection::extra, ParameterTraits::optional | ParameterTraits::custom, std::wstring(), std::wstring()});
+				ret.emplace_back(ParameterTraits{"accelerate", ParameterSection::extra, ParameterTraits::optional | ParameterTraits::custom, std::wstring(), std::wstring()});
 				return ret;
 			}();
 			return ret;
 		}
 	case GOOGLE_DRIVE:
 	case ONEDRIVE:
-	{
-		static std::vector<ParameterTraits> const ret = []() {
-			std::vector<ParameterTraits> ret;
-			ret.emplace_back(ParameterTraits{"login_hint", ParameterSection::user, ParameterTraits::optional, std::wstring(), _("Name or email address")});
-			ret.emplace_back(ParameterTraits{"oauth_identity", ParameterSection::extra, ParameterTraits::optional| ParameterTraits::custom, std::wstring(), std::wstring()});
+		{
+			static std::vector<ParameterTraits> const ret = []() {
+				std::vector<ParameterTraits> ret;
+				ret.emplace_back(ParameterTraits{"login_hint", ParameterSection::user, ParameterTraits::optional, std::wstring(), _("Name or email address")});
+				ret.emplace_back(ParameterTraits{"oauth_identity", ParameterSection::extra, ParameterTraits::optional| ParameterTraits::custom, std::wstring(), std::wstring()});
+				return ret;
+			}();
 			return ret;
-		}();
-		return ret;
-	}
+		}
 	case DROPBOX:
-	{
-		static std::vector<ParameterTraits> const ret = []() {
-			std::vector<ParameterTraits> ret;
-			ret.emplace_back(ParameterTraits{"oauth_identity", ParameterSection::extra, ParameterTraits::optional | ParameterTraits::custom, std::wstring(), std::wstring()});
-			ret.emplace_back(ParameterTraits{"root_namespace", ParameterSection::extra, ParameterTraits::optional | ParameterTraits::custom, std::wstring(), std::wstring()});
+		{
+			static std::vector<ParameterTraits> const ret = []() {
+				std::vector<ParameterTraits> ret;
+				ret.emplace_back(ParameterTraits{"oauth_identity", ParameterSection::extra, ParameterTraits::optional | ParameterTraits::custom, std::wstring(), std::wstring()});
+				ret.emplace_back(ParameterTraits{"root_namespace", ParameterSection::extra, ParameterTraits::optional | ParameterTraits::custom, std::wstring(), std::wstring()});
+				return ret;
+			}();
 			return ret;
-		}();
-		return ret;
-	}
+		}
 	case BOX:
-	{
-		static std::vector<ParameterTraits> const ret = []() {
-			std::vector<ParameterTraits> ret;
-			ret.emplace_back(ParameterTraits{"oauth_identity", ParameterSection::extra, ParameterTraits::optional | ParameterTraits::custom, std::wstring(), std::wstring()});
+		{
+			static std::vector<ParameterTraits> const ret = []() {
+				std::vector<ParameterTraits> ret;
+				ret.emplace_back(ParameterTraits{"oauth_identity", ParameterSection::extra, ParameterTraits::optional | ParameterTraits::custom, std::wstring(), std::wstring()});
+				return ret;
+			}();
 			return ret;
-		}();
-		return ret;
-	}
+		}
 	case STORJ:
-	{
-		static std::vector<ParameterTraits> const ret = []() {
-			std::vector<ParameterTraits> ret;
-			ret.emplace_back(ParameterTraits{"passphrase_hash", ParameterSection::extra, ParameterTraits::optional | ParameterTraits::custom, std::wstring(), std::wstring()});
+		{
+			static std::vector<ParameterTraits> const ret = []() {
+				std::vector<ParameterTraits> ret;
+				ret.emplace_back(ParameterTraits{"passphrase_hash", ParameterSection::extra, ParameterTraits::optional | ParameterTraits::custom, std::wstring(), std::wstring()});
+				return ret;
+			}();
 			return ret;
-		}();
-		return ret;
-	}
+		}
 	case STORJ_GRANT:
-	{
-		static std::vector<ParameterTraits> const ret = []() {
-			std::vector<ParameterTraits> ret;
-			ret.emplace_back(ParameterTraits{"credentials_hash", ParameterSection::extra, ParameterTraits::optional | ParameterTraits::custom, std::wstring(), std::wstring()});
+		{
+			static std::vector<ParameterTraits> const ret = []() {
+				std::vector<ParameterTraits> ret;
+				ret.emplace_back(ParameterTraits{"credentials_hash", ParameterSection::extra, ParameterTraits::optional | ParameterTraits::custom, std::wstring(), std::wstring()});
+				return ret;
+			}();
 			return ret;
-		}();
-		return ret;
-	}
+		}
+
+	case CLOUDFLARE_R2:
+		{
+			static std::vector<ParameterTraits> const ret = []() {
+				std::vector<ParameterTraits> ret;
+				ret.emplace_back(ParameterTraits{"identuser", ParameterSection::user, ParameterTraits::optional, std::wstring(), std::wstring()});
+				ret.emplace_back(ParameterTraits{"jurisdiction", ParameterSection::extra, ParameterTraits::optional | ParameterTraits::custom, std::wstring(), std::wstring()});
+				return ret;
+			}();
+			return ret;
+		}
+
 	default:
 		break;
 	}
@@ -1036,10 +1088,12 @@ std::tuple<std::wstring, std::wstring> GetDefaultHost(ServerProtocol protocol)
 	case AZURE_BLOB:
 		return std::tuple<std::wstring, std::wstring>{L"blob.core.windows.net", L""};
 	case GOOGLE_CLOUD:
+	case GOOGLE_CLOUD_SVC_ACC:
 		return std::tuple<std::wstring, std::wstring>{L"storage.googleapis.com", L""};
 	case GOOGLE_DRIVE:
 		return std::tuple<std::wstring, std::wstring>{L"www.googleapis.com", L""};
 	case S3:
+	case S3_SSO:
 		return std::tuple<std::wstring, std::wstring>{L"s3.amazonaws.com", L""};
 	case DROPBOX:
 		return std::tuple<std::wstring, std::wstring>{L"api.dropboxapi.com", L""};
@@ -1054,6 +1108,8 @@ std::tuple<std::wstring, std::wstring> GetDefaultHost(ServerProtocol protocol)
 	case STORJ:
 	case STORJ_GRANT:
 		return std::tuple<std::wstring, std::wstring>{L"us-central-1.tardigrade.io", L""};
+	case CLOUDFLARE_R2:
+		return std::tuple<std::wstring, std::wstring>{ L"r2.cloudflarestorage.com", L"" };
 	default:
 		break;
 	}
@@ -1063,7 +1119,8 @@ std::tuple<std::wstring, std::wstring> GetDefaultHost(ServerProtocol protocol)
 
 bool ProtocolHasUser(ServerProtocol protocol)
 {
-	return protocol != DROPBOX && protocol != ONEDRIVE && protocol != BOX && protocol != GOOGLE_DRIVE && protocol != STORJ_GRANT;
+	return protocol != DROPBOX && protocol != ONEDRIVE && protocol != BOX &&
+		   protocol != GOOGLE_DRIVE && protocol != STORJ_GRANT && protocol != GOOGLE_CLOUD_SVC_ACC;
 }
 
 bool CServer::SameResource(CServer const& other) const
