@@ -49,7 +49,13 @@ enum ServerProtocol
 
 	STORJ_GRANT,
 
-	MAX_VALUE = STORJ_GRANT
+	S3_SSO,
+
+	GOOGLE_CLOUD_SVC_ACC,
+
+	CLOUDFLARE_R2,
+
+	MAX_VALUE = CLOUDFLARE_R2
 };
 
 enum ServerType
@@ -109,9 +115,11 @@ enum class ProtocolFeature
 	Security, // Encryption, integrity protection and authentication
 	UnixChmod,
 	ProExclusive,
+	PruneOldVersions,
 	ListVersions,
 	DownloadVersion,
-	DeleteVersion
+	DeleteVersion,
+	Share
 };
 
 enum class CaseSensitivity
@@ -242,10 +250,12 @@ enum class LogonType
 	interactive,
 	account,
 	key,
-	profile,
+	profile,		// S3 profile
+	adc,			// Google Cloud Application Default Credenttials
 
 	count
 };
+
 std::wstring FZC_PUBLIC_SYMBOL GetNameFromLogonType(LogonType type);
 LogonType FZC_PUBLIC_SYMBOL GetLogonTypeFromName(std::wstring const& name);
 
