@@ -366,6 +366,7 @@ bool CLocalListView::DisplayDir(CLocalPath const& dirname)
 		data.dir = true;
 		data.name = _T("..");
 		data.size = -1;
+		data.attributes = {};
 		m_fileData.push_back(data);
 		m_indexMapping.push_back(0);
 	}
@@ -478,7 +479,7 @@ regular_dir:
 		RefreshComparison();
 	}
 
-	ReselectItems(selectedNames, focused, focusedItem, ensureVisible);
+	ReselectItems(selectedNames, std::move(focused), focusedItem, ensureVisible);
 
 	RefreshListOnly();
 
@@ -962,7 +963,7 @@ void CLocalListView::OnMenuUpload(wxCommandEvent& event)
 // Create a new Directory
 void CLocalListView::OnMenuMkdir(wxCommandEvent&)
 {
-	wxString newdir = MenuMkdir();
+	CLocalPath newdir = MenuMkdir();
 	if (!newdir.empty()) {
 		m_state.RefreshLocal();
 	}
@@ -971,7 +972,7 @@ void CLocalListView::OnMenuMkdir(wxCommandEvent&)
 // Create a new Directory and enter the new Directory
 void CLocalListView::OnMenuMkdirChgDir(wxCommandEvent&)
 {
-	std::wstring newdir = MenuMkdir().ToStdWstring();
+	CLocalPath newdir = MenuMkdir();
 	if (newdir.empty()) {
 		return;
 	}
@@ -990,39 +991,33 @@ void CLocalListView::OnMenuMkdirChgDir(wxCommandEvent&)
 
 // Helper-Function to create a new Directory
 // Returns the name of the new directory
-wxString CLocalListView::MenuMkdir()
+CLocalPath CLocalListView::MenuMkdir()
 {
 	CInputDialog dlg;
 	if (!dlg.Create(this, _("Create directory"), _("Please enter the name of the directory which should be created:"))) {
-		return wxString();
+		return {};
 	}
 
 	if (dlg.ShowModal() != wxID_OK) {
-		return wxString();
+		return {};
 	}
 
 	if (dlg.GetValue().empty()) {
 		wxBell();
-		return wxString();
+		return {};
 	}
 
-	wxFileName fn(dlg.GetValue(), wxString());
-	fn.Normalize(wxPATH_NORM_ALL, m_dir.GetPath());
-
-	bool res;
-
-	{
-		wxLogNull log;
-		res = fn.Mkdir(fn.GetPath(), 0777, wxPATH_MKDIR_FULL);
-	}
-
-	if (!res) {
+	CLocalPath p = m_dir;
+	if (!p.ChangePath(dlg.GetValue().ToStdWstring()) || !p.IsWriteable()) {
 		wxBell();
-		return wxString();
+		return {};
+	}
+	if (!fz::mkdir(fz::to_native(p.GetPath()), true)) {
+		wxBell();
+		return {};
 	}
 
-	// Return name of the New Directory
-	return fn.GetPath();
+	return p;
 }
 
 void CLocalListView::OnMenuDelete(wxCommandEvent&)
@@ -1115,7 +1110,7 @@ void CLocalListView::OnKeyDown(wxKeyEvent& event)
 	}
 	else if (code == 'F' && event.GetModifiers() == wxMOD_CONTROL) {
 		if (m_parentView) {
-			m_parentView->ShowSearchPanel();
+			m_parentView->ShowSearchPanel(options_);
 		}
 	}
 	else {
@@ -1252,7 +1247,7 @@ void CLocalListView::ApplyCurrentFilter()
 		RefreshComparison();
 	}
 
-	ReselectItems(selectedNames, focused, focusedItem);
+	ReselectItems(selectedNames, std::move(focused), focusedItem);
 
 	if (!IsComparing()) {
 		RefreshListOnly();
@@ -1466,7 +1461,7 @@ void CLocalListView::OnBeginDrag(wxListEvent&)
 	pDragDropManager->localParent = m_dir;
 	pDragDropManager->dragDataObject = &obj;
 
-	auto const path = m_dir.GetPath();
+	auto const& path = m_dir.GetPath();
 
 	bool added = false;
 
@@ -1719,6 +1714,7 @@ void CLocalListView::StartComparison()
 		data.dir = false;
 		data.icon = -1;
 		data.size = -1;
+		data.attributes = {};
 		data.comparison_flags = fill;
 		m_fileData.push_back(data);
 	}
@@ -1951,11 +1947,17 @@ void CLocalListView::OnMenuOpen(wxCommandEvent&)
 
 bool CLocalListView::ItemIsDir(int index) const
 {
+	if (index < 0 || static_cast<size_t>(index) >= m_fileData.size()) {
+		return false;
+	}
 	return m_fileData[index].dir;
 }
 
 int64_t CLocalListView::ItemGetSize(int index) const
 {
+	if (index < 0 || static_cast<size_t>(index) >= m_fileData.size()) {
+		return -1;
+	}
 	return m_fileData[index].size;
 }
 
