@@ -2105,29 +2105,56 @@ bool CMainFrame::ConnectToSite(Site & data, Bookmark const& bookmark, CState* pS
 		int action = options_.get_int(OPTION_ALREADYCONNECTED_CHOICE);
 		if (action < 2) {
 			wxDialogEx dlg;
-			if (!dlg.Load(this, _T("ID_ALREADYCONNECTED"))) {
+			if (!dlg.Create(this, nullID, _("Already connected"))) {
 				return false;
 			}
 
+			auto & lay = dlg.layout();
+			auto top = lay.createMain(&dlg, 2);
+
+			top->Add(new wxStaticBitmap(&dlg, nullID, wxArtProvider::GetBitmap(wxART_QUESTION, wxART_OTHER, wxSize(32, 32))), 0, wxALL, lay.dlgUnits(2));
+
+			auto main = lay.createFlex(1);
+			top->Add(main);
+
+			main->Add(new wxStaticText(&dlg, nullID, _("FileZilla is already connected to a server.")));
+
+			auto newtab = new wxRadioButton(&dlg, nullID, _("Establish connection in a new tab"), wxDefaultPosition, wxDefaultSize, wxRB_GROUP);
+			main->Add(newtab);
+			auto oldtab = new wxRadioButton(&dlg, nullID, _("&Abort previous connection and connect in current tab"));
+			main->Add(oldtab);
+
+			main->AddSpacer(0);
+
+			auto remember = new wxCheckBox(&dlg, nullID, _("Al&ways perform this action"));
+			main->Add(remember);
+
 			if (action != 0) {
-				XRCCTRL(dlg, "ID_OLDTAB", wxRadioButton)->SetValue(true);
+				oldtab->SetValue(true);
+				oldtab->SetFocus();
 			}
 			else {
-				XRCCTRL(dlg, "ID_NEWTAB", wxRadioButton)->SetValue(true);
+				newtab->SetValue(true);
+				newtab->SetFocus();
 			}
 
+			lay.createButtonSizerButtons(&dlg, main, false);
+
+			dlg.GetSizer()->Fit(&dlg);
+			dlg.GetSizer()->SetSizeHints(&dlg);
+
 			if (dlg.ShowModal() != wxID_OK) {
 				return false;
 			}
 
-			if (XRCCTRL(dlg, "ID_NEWTAB", wxRadioButton)->GetValue()) {
+			if (newtab->GetValue()) {
 				action = 0;
 			}
 			else {
 				action = 1;
 			}
 
-			if (XRCCTRL(dlg, "ID_REMEMBER", wxCheckBox)->IsChecked()) {
+			if (remember->IsChecked()) {
 				action |= 2;
 			}
 			options_.set(OPTION_ALREADYCONNECTED_CHOICE, action);
