@@ -2537,7 +2537,7 @@ void CQueueView::OnTimer(wxTimerEvent& event)
 			pData->m_idleDisconnectTimer = 0;
 
 			if (pData->pEngine->IsConnected()) {
-				pData->pEngine->Execute(CDisconnectCommand());
+				(void)pData->pEngine->Execute(CDisconnectCommand());
 			}
 		}
 	}
