@@ -1426,15 +1426,33 @@ void CRemoteListView::OnMenuDelete(wxCommandEvent&)
 	bool follow_symlink = false;
 	if (selected_link) {
 		wxDialogEx dlg;
-		if (!dlg.Load(this, _T("ID_DELETE_SYMLINK"))) {
-			wxBell();
+
+		if (!dlg.Create(this, nullID, _("Deleting symbolic link"))) {
 			return;
 		}
+		auto & lay = dlg.layout();
+		auto main = lay.createMain(&dlg, 1);
+
+		main->Add(new wxStaticText(&dlg, nullID, _("You have selected at least one symbolic link.")));
+		main->Add(new wxStaticText(&dlg, nullID, _("If the symlink points to a directory, FileZilla can either delete the symbolic link or remove the contents of the linked directory.")));
+		main->Add(new wxStaticText(&dlg, nullID, _("Please select an action:")));
+
+		auto dele = new wxRadioButton(&dlg, nullID, _("&Delete symlink"), wxDefaultPosition, wxDefaultSize, wxRB_GROUP);
+		main->Add(dele);
+		dele->SetValue(1);
+		dele->SetFocus();
+		auto recurse = new wxRadioButton(&dlg, nullID, _("&Follow symlink, delete target directory contents"));
+		main->Add(recurse);
+		main->Add(new wxStaticText(&dlg, nullID, _("Symlinks to files are not affected by this choice, such symlinks are always deleted.")));
+
+		lay.createButtonSizerButtons(&dlg, main, true);
+		dlg.GetSizer()->Fit(&dlg);
+
 		if (dlg.ShowModal() != wxID_OK) {
 			return;
 		}
 
-		follow_symlink = XRCCTRL(dlg, "ID_RECURSE", wxRadioButton)->GetValue();
+		follow_symlink = recurse->GetValue();
 	}
 
 	CFilterManager filter;
