@@ -710,14 +710,6 @@ void CEditHandler::CheckForModifications()
 
 void CEditHandler::DoCheckForModifications()
 {
-#ifdef __WXMSW__
-	// Don't check for changes if mouse is captured,
-	// e.g. if user is dragging a file
-	if (GetCapture()) {
-		return;
-	}
-#endif
-
 	static bool insideCheckForModifications = false;
 	if (insideCheckForModifications) {
 		return;
@@ -942,7 +934,7 @@ void CEditHandler::SetTimerState()
 		}
 	}
 	else if (editing) {
-		m_timer.Start(15000);
+		m_timer.Start(10000);
 	}
 }
 
