@@ -1650,7 +1650,13 @@ CQueue::CQueue(wxWindow* parent, CMainFrame *pMainFrame, CAsyncRequestQueue *pAs
 
 	m_pQueueView->LoadQueue();
 
-	Bind(wxEVT_COMMAND_AUINOTEBOOK_TAB_RIGHT_UP, [this](wxAuiNotebookEvent& ev) {
+#if FZ_WINDOWS
+	auto evt = wxEVT_COMMAND_AUINOTEBOOK_TAB_RIGHT_UP;
+#else
+	auto evt = wxEVT_COMMAND_AUINOTEBOOK_TAB_RIGHT_DOWN;
+#endif
+
+	Bind(evt, [this](wxAuiNotebookEvent& ev) {
 		int tab = ev.GetSelection();
 
 		wxContextMenuEvent cev(wxEVT_CONTEXT_MENU);
