@@ -87,8 +87,8 @@ GeneralSiteControls::GeneralSiteControls(wxWindow & parent, DialogLayout const&
 
 	lay.gbNewRow(bag);
 	lay.gbAdd(bag, new wxStaticText(&parent, nullID, _("Pro&tocol:")), lay.valign);
-	auto protocols = new wxChoice(&parent, XRCID("ID_PROTOCOL"));
-	lay.gbAdd(bag, protocols, lay.valigng);
+	protocol_choice_ = new wxChoice(&parent, nullID);
+	lay.gbAdd(bag, protocol_choice_, lay.valigng);
 
 	lay.gbNewRow(bag);
 	lay.gbAdd(bag, new wxStaticText(&parent, XRCID("ID_HOST_DESC"), _("&Host:")), lay.valign);
@@ -96,16 +96,28 @@ GeneralSiteControls::GeneralSiteControls(wxWindow & parent, DialogLayout const&
 	row->AddGrowableCol(0);
 	lay.gbAdd(bag, row, lay.valigng);
 	row->Add(new wxTextCtrlEx(&parent, XRCID("ID_HOST")), lay.valigng);
-	row->Add(new wxStaticText(&parent, nullID, _("&Port:")), lay.valign);
+	row->Add(new wxStaticText(&parent, XRCID("ID_PORT_DESC"), _("&Port:")), lay.valign);
 	auto* port = new wxTextCtrlEx(&parent, XRCID("ID_PORT"), wxString(), wxDefaultPosition, wxSize(lay.dlgUnits(27), -1));
 	port->SetMaxLength(5);
 	row->Add(port, lay.valign);
 
 	lay.gbNewRow(bag);
-	lay.gbAdd(bag, new wxStaticText(&parent, XRCID("ID_ENCRYPTION_DESC"), _("&Encryption:")), lay.valign);
+	lay.gbAdd(bag, new wxStaticText(&parent, XRCID("ID_JURISDICTION_DESC"), _("&Jurisdiction:")), lay.valign);
+
+	auto jrow = lay.createFlex(0, 1);
+	jrow->AddGrowableCol(0);
+	lay.gbAdd(bag, jrow, lay.valigng);
+
+	auto jurisdictions = new wxChoice(&parent, XRCID("ID_JURISDICTION"));
+	jrow->Add(jurisdictions, lay.valigng);
+
+	lay.gbNewRow(bag);
+	encryption_desc_ = new wxStaticText(&parent, nullID, _("&Encryption:"));
+	lay.gbAdd(bag, encryption_desc_, lay.valign);
 	auto brow = new wxBoxSizer(wxHORIZONTAL);
 	lay.gbAdd(bag, brow, lay.valigng);
-	brow->Add(new wxChoice(&parent, XRCID("ID_ENCRYPTION")), 1);
+	encryption_ = new wxChoice(&parent, nullID);
+	brow->Add(encryption_, 1);
 	brow->Add(new wxHyperlinkCtrl(&parent, XRCID("ID_DOCS"), _("Docs"), L"https://docs.storj.io/how-tos/set-up-filezilla-for-decentralized-file-transfer"), 0, wxRIGHT | wxALIGN_CENTER_VERTICAL, lay.dlgUnits(2))->Show(false);
 	brow->Add(new wxHyperlinkCtrl(&parent, XRCID("ID_SIGNUP"), _("Signup"), L"https://storj.io/signup/?partner=filezilla"), lay.valign)->Show(false);
 	brow->AddSpacer(0);
@@ -163,22 +175,25 @@ GeneralSiteControls::GeneralSiteControls(wxWindow & parent, DialogLayout const&
 		auto const entry = findGroup(proto);
 		if (entry.first != protocolGroups().cend()) {
 			if (entry.second == entry.first->protocols.cbegin()) {
-				mainProtocolListIndex_[proto] = protocols->Append(entry.first->name);
+				mainProtocolListIndex_[proto] = protocol_choice_->Append(entry.first->name);
 			}
 			else {
 				mainProtocolListIndex_[proto] = mainProtocolListIndex_[entry.first->protocols.front().first];
 			}
 		}
 		else {
-			mainProtocolListIndex_[proto] = protocols->Append(CServer::GetProtocolName(proto));
+			mainProtocolListIndex_[proto] = protocol_choice_->Append(CServer::GetProtocolName(proto));
 		}
 	}
+	if (protocol_choice_->GetCount()) {
+		protocol_choice_->Select(0);
+	}
 
 	for (int i = 0; i < static_cast<int>(LogonType::count); ++i) {
 		logonTypes->Append(GetNameFromLogonType(static_cast<LogonType>(i)));
 	}
 
-	protocols->Bind(wxEVT_CHOICE, [this](wxEvent const&) {
+	protocol_choice_->Bind(wxEVT_CHOICE, [this](wxEvent const&) {
 		auto p = GetProtocol();
 		SetProtocol(p);
 		if (changeHandler_) {
@@ -186,6 +201,11 @@ GeneralSiteControls::GeneralSiteControls(wxWindow & parent, DialogLayout const&
 		}
 	});
 
+	jurisdictions->Append(_("North America"));
+	jurisdictions->Append(_("Europe Union"));
+	jurisdictions->Append(_("FedRAMP"));
+	jurisdictions->Select(0);
+
 	logonTypes->Bind(wxEVT_CHOICE, [this](wxEvent const&) {
 		if (changeHandler_) {
 			changeHandler_(GetProtocol(), GetLogonType());
@@ -198,25 +218,48 @@ GeneralSiteControls::GeneralSiteControls(wxWindow & parent, DialogLayout const&
 #else
 		wchar_t const* all = L"*";
 #endif
-		wxString wildcards = wxString::Format(L"%s|*.ppk|%s|*.pem|%s|%s",
+		wxString wildcards;
+		auto protocol = GetProtocol();
+
+		if (protocol == SFTP) {
+			wildcards = wxString::Format(L"%s|*.ppk|%s|*.pem|%s|%s",
 											  _("PPK files"), _("PEM files"), _("All files"), all);
+		}
+		if (protocol == GOOGLE_CLOUD_SVC_ACC) {
+			wildcards = wxString::Format(L"%s|*.json|%s|%s", _("JSON files"), _("All files"), all);
+		}
+
 		wxFileDialog dlg(&parent_, _("Choose a key file"), wxString(), wxString(), wildcards, wxFD_OPEN|wxFD_FILE_MUST_EXIST);
 
 		if (dlg.ShowModal() == wxID_OK) {
 			std::wstring keyFilePath = dlg.GetPath().ToStdWstring();
-			// If the selected file was a PEM file, LoadKeyFile() will automatically convert it to PPK
-			// and tell us the new location.
-			CFZPuttyGenInterface fzpg(&parent_);
 
-			std::wstring keyFileComment, keyFileData;
-			if (fzpg.LoadKeyFile(keyFilePath, false, keyFileComment, keyFileData)) {
-				XRCCTRL(parent_, "ID_KEYFILE", wxTextCtrl)->ChangeValue(keyFilePath);
-	#if USE_MAC_SANDBOX
-				OSXSandboxUserdirs::Get().AddFile(keyFilePath);
-	#endif
+			if (protocol == SFTP) {
+				// If the selected file was a PEM file, LoadKeyFile() will automatically convert it to PPK
+				// and tell us the new location.
+				CFZPuttyGenInterface fzpg(&parent_);
+
+				std::wstring keyFileComment, keyFileData;
+				if (fzpg.LoadKeyFile(keyFilePath, false, keyFileComment, keyFileData)) {
+					XRCCTRL(parent_, "ID_KEYFILE", wxTextCtrl)->ChangeValue(keyFilePath);
+#if USE_MAC_SANDBOX
+					OSXSandboxUserdirs::Get().AddFile(keyFilePath);
+#endif
+				}
+				else {
+					xrc_call(parent_, "ID_KEYFILE", &wxWindow::SetFocus);
+				}
 			}
-			else {
-				xrc_call(parent_, "ID_KEYFILE", &wxWindow::SetFocus);
+			if (protocol == GOOGLE_CLOUD_SVC_ACC) {
+				if (ValidateGoogleAccountKeyFile(keyFilePath)) {
+					XRCCTRL(parent_, "ID_KEYFILE", wxTextCtrl)->ChangeValue(keyFilePath);
+#if USE_MAC_SANDBOX
+					OSXSandboxUserdirs::Get().AddFile(keyFilePath);
+#endif
+				}
+				else {
+					xrc_call(parent_, "ID_KEYFILE", &wxWindow::SetFocus);
+				}
 			}
 		}
 	});
@@ -236,6 +279,8 @@ void GeneralSiteControls::SetSite(Site const& site)
 		xrc_call(parent_, "ID_KEYFILE", &wxTextCtrl::ChangeValue, wxString());
 
 		xrc_call(parent_, "ID_LOGONTYPE", &wxChoice::SetStringSelection, GetNameFromLogonType(logonType_));
+
+		xrc_call(parent_, "ID_JURISDICTION", &wxChoice::SetSelection, 0);
 	}
 	else {
 		xrc_call(parent_, "ID_HOST", &wxTextCtrl::ChangeValue, site.Format(ServerFormat::host_only));
@@ -258,7 +303,8 @@ void GeneralSiteControls::SetSite(Site const& site)
 
 		std::wstring pass = site.credentials.GetPass();
 
-		if (logonType_ != LogonType::anonymous && logonType_ != LogonType::interactive && (protocol != SFTP || logonType_ != LogonType::key)) {
+		if (logonType_ != LogonType::anonymous && logonType_ != LogonType::interactive &&
+			(protocol != SFTP || logonType_ != LogonType::key) && protocol != GOOGLE_CLOUD_SVC_ACC) {
 			if (site.credentials.encrypted_) {
 				xrc_call(parent_, "ID_PASS", &wxTextCtrl::ChangeValue, wxString());
 
@@ -306,6 +352,22 @@ void GeneralSiteControls::SetSite(Site const& site)
 		}
 
 		xrc_call(parent_, "ID_KEYFILE", &wxTextCtrl::ChangeValue, site.credentials.keyFile_);
+
+		if (site.server.GetProtocol() == CLOUDFLARE_R2) {
+			auto region = site.server.GetExtraParameter("jurisdiction");
+			if (region.empty()) {
+				xrc_call(parent_, "ID_JURISDICTION", &wxChoice::SetSelection, 0);
+			}
+			else if (region == L"eu") {
+				xrc_call(parent_, "ID_JURISDICTION", &wxChoice::SetSelection, 1);
+			}
+			else if (region == L"fedramp") {
+				xrc_call(parent_, "ID_JURISDICTION", &wxChoice::SetSelection, 2);
+			}
+			else {
+				xrc_call(parent_, "ID_JURISDICTION", &wxChoice::SetSelection, -1);
+			}
+		}
 	}
 }
 
@@ -317,8 +379,8 @@ void GeneralSiteControls::SetControlVisibility(ServerProtocol protocol, LogonTyp
 	auto const group = findGroup(protocol);
 	bool const isFtp = group.first != protocolGroups().cend() && group.first->protocols.front().first == FTP;
 
-	xrc_call(parent_, "ID_ENCRYPTION_DESC", &wxStaticText::Show, group.first != protocolGroups().cend());
-	xrc_call(parent_, "ID_ENCRYPTION", &wxChoice::Show, group.first != protocolGroups().cend());
+	encryption_desc_->Show(group.first != protocolGroups().cend());
+	encryption_->Show(group.first != protocolGroups().cend());
 
 	xrc_call(parent_, "ID_DOCS", &wxControl::Show, protocol == STORJ || protocol == STORJ_GRANT);
 	xrc_call(parent_, "ID_SIGNUP", &wxControl::Show, protocol == STORJ || protocol == STORJ_GRANT);
@@ -340,9 +402,18 @@ void GeneralSiteControls::SetControlVisibility(ServerProtocol protocol, LogonTyp
 		}
 	}
 
-	bool const hasUser = ProtocolHasUser(protocol) && type != LogonType::anonymous;
-	bool const hasPw = type != LogonType::anonymous && type != LogonType::interactive && (protocol != SFTP || type != LogonType::key) && (protocol != S3 || type != LogonType::profile);
+	xrc_call(parent_, "ID_HOST_DESC", &wxStaticText::Show, protocol != CLOUDFLARE_R2);
+	xrc_call(parent_, "ID_HOST", &wxStaticText::Show, protocol != CLOUDFLARE_R2);
+	xrc_call(parent_, "ID_PORT_DESC", &wxStaticText::Show, protocol != CLOUDFLARE_R2);
+	xrc_call(parent_, "ID_PORT", &wxStaticText::Show, protocol != CLOUDFLARE_R2);
+	xrc_call(parent_, "ID_JURISDICTION_DESC", &wxChoice::Show, protocol == CLOUDFLARE_R2);
+	xrc_call(parent_, "ID_JURISDICTION", &wxChoice::Show, protocol == CLOUDFLARE_R2);
 
+	bool const hasUser = ProtocolHasUser(protocol) && type != LogonType::anonymous;
+	bool const hasPw = type != LogonType::anonymous && type != LogonType::interactive &&
+		(protocol != SFTP || type != LogonType::key) &&
+		(!(protocol == S3 || protocol == CLOUDFLARE_R2) || type != LogonType::profile) &&
+		protocol != GOOGLE_CLOUD_SVC_ACC;
 
 	xrc_call(parent_, "ID_USER_DESC", &wxStaticText::Show, hasUser);
 	xrc_call(parent_, "ID_USER", &wxTextCtrl::Show, hasUser);
@@ -350,9 +421,9 @@ void GeneralSiteControls::SetControlVisibility(ServerProtocol protocol, LogonTyp
 	xrc_call(parent_, "ID_PASS", &wxTextCtrl::Show, hasPw);
 	xrc_call(parent_, "ID_ACCOUNT_DESC", &wxStaticText::Show, isFtp && type == LogonType::account);
 	xrc_call(parent_, "ID_ACCOUNT", &wxTextCtrl::Show, isFtp && type == LogonType::account);
-	xrc_call(parent_, "ID_KEYFILE_DESC", &wxStaticText::Show, protocol == SFTP && type == LogonType::key);
-	xrc_call(parent_, "ID_KEYFILE", &wxTextCtrl::Show, protocol == SFTP && type == LogonType::key);
-	xrc_call(parent_, "ID_KEYFILE_BROWSE", &wxButton::Show, protocol == SFTP && type == LogonType::key);
+	xrc_call(parent_, "ID_KEYFILE_DESC", &wxStaticText::Show, (protocol == SFTP || protocol == GOOGLE_CLOUD_SVC_ACC) && type == LogonType::key);
+	xrc_call(parent_, "ID_KEYFILE", &wxTextCtrl::Show, (protocol == SFTP || protocol == GOOGLE_CLOUD_SVC_ACC) && type == LogonType::key);
+	xrc_call(parent_, "ID_KEYFILE_BROWSE", &wxButton::Show, (protocol == SFTP || protocol == GOOGLE_CLOUD_SVC_ACC) && type == LogonType::key);
 
 	wxString hostLabel = _("&Host:");
 	wxString hostHint;
@@ -375,13 +446,24 @@ void GeneralSiteControls::SetControlVisibility(ServerProtocol protocol, LogonTyp
 		passLabel = _("Access &Grant:");
 		break;
 	case S3:
+	case CLOUDFLARE_R2:
 		// @translator: Keep short
 		userLabel = _("&Access key ID:");
 		if (type == LogonType::profile) {
-			userLabel = _("&Profile");
+			userLabel = _("&Profile:");
 		}
 		// @translator: Keep short
 		passLabel = _("Secret Access &Key:");
+		if (protocol == CLOUDFLARE_R2) {
+			hostLabel = _("Jurisdiction:");
+		}
+		break;
+	case S3_SSO:
+		// @translator: Keep short
+		userLabel = _("&Account ID:");
+		if (type == LogonType::profile) {
+			userLabel = _("&Profile:");
+		}
 		break;
 	case AZURE_FILE:
 	case AZURE_BLOB:
@@ -404,7 +486,7 @@ void GeneralSiteControls::SetControlVisibility(ServerProtocol protocol, LogonTyp
 		break;
 	case B2:
 		// @translator: Keep short
-		userLabel = _("&Account ID:");
+		userLabel = _("&Application Key ID:");
 		// @translator: Keep short
 		passLabel = _("Application &Key:");
 		break;
@@ -451,7 +533,7 @@ void GeneralSiteControls::SetControlVisibility(ServerProtocol protocol, LogonTyp
 		return rows.end() - 1;
 	};
 
-	auto SetLabel = [](wxStaticText & label, ServerProtocol const, std::string const& name) {
+	auto SetLabel = [](wxStaticText & label, ServerProtocol const protocol, std::string const& name) {
 		if (name == "login_hint") {
 			label.SetLabel(_("Login (optional):"));
 		}
@@ -460,7 +542,12 @@ void GeneralSiteControls::SetControlVisibility(ServerProtocol protocol, LogonTyp
 			label.SetLabel(_("Identity service path:"));
 		}
 		else if (name == "identuser") {
-			label.SetLabel(_("&User:"));
+			if (protocol == CLOUDFLARE_R2) {
+				label.SetLabel(_("Acco&unt ID:"));
+			}
+			else {
+				label.SetLabel(_("&User:"));
+			}
 		}
 		else {
 			label.SetLabel(fz::to_wstring_from_utf8(name));
@@ -519,7 +606,7 @@ void GeneralSiteControls::SetControlVisibility(ServerProtocol protocol, LogonTyp
 		++it;
 	}
 
-	auto encSizer = xrc_call(parent_, "ID_ENCRYPTION", &wxWindow::GetContainingSizer);
+	auto encSizer = encryption_->GetContainingSizer();
 	encSizer->Show(encSizer->GetItemCount() - 1, paramIt[ParameterSection::host] == extraParameters_[ParameterSection::host].cbegin());
 
 	for (int i = 0; i < ParameterSection::section_count; ++i) {
@@ -618,7 +705,7 @@ bool GeneralSiteControls::UpdateSite(Site & site, bool silent)
 		user.clear();
 	}
 	else if (logon_type != LogonType::ask &&
-	         logon_type != LogonType::interactive &&
+	         (logon_type != LogonType::interactive || protocol == S3_SSO) &&
 	         user.empty())
 	{
 		// Require username for non-anonymous, non-ask logon type
@@ -674,15 +761,26 @@ bool GeneralSiteControls::UpdateSite(Site & site, bool silent)
 			return false;
 		}
 
-		// Check (again) that the key file is in the correct format since it might have been changed in the meantime
-		CFZPuttyGenInterface cfzg(wxGetTopLevelParent(&parent_));
+		if (protocol == SFTP) {
+			// Check (again) that the key file is in the correct format since it might have been changed in the meantime
+			CFZPuttyGenInterface cfzg(wxGetTopLevelParent(&parent_));
 
-		std::wstring keyFileComment, keyFileData;
-		if (!cfzg.LoadKeyFile(keyFile, silent, keyFileComment, keyFileData)) {
-			if (!silent) {
-				xrc_call(parent_, "ID_KEYFILE", &wxWindow::SetFocus);
+			std::wstring keyFileComment, keyFileData;
+			if (!cfzg.LoadKeyFile(keyFile, silent, keyFileComment, keyFileData)) {
+				if (!silent) {
+					xrc_call(parent_, "ID_KEYFILE", &wxWindow::SetFocus);
+				}
+				return false;
+			}
+		}
+
+		if (protocol == GOOGLE_CLOUD_SVC_ACC) {
+			if (!ValidateGoogleAccountKeyFile(keyFile)) {
+				if (!silent) {
+					xrc_call(parent_, "ID_KEYFILE", &wxWindow::SetFocus);
+				}
+				return false;
 			}
-			return false;
 		}
 
 		site.credentials.keyFile_ = keyFile;
@@ -759,6 +857,22 @@ bool GeneralSiteControls::UpdateSite(Site & site, bool silent)
 		}
 #endif
 
+	if (site.server.GetProtocol() == CLOUDFLARE_R2) {
+		auto jurisdiction = xrc_call(parent_, "ID_JURISDICTION", &wxChoice::GetSelection);
+		if (jurisdiction == wxNOT_FOUND || jurisdiction == 0) {
+			site.server.ClearExtraParameter("jurisdiction");
+			site.server.SetHost(L"r2.cloudflarestorage.com", site.server.GetPort());
+		}
+		else if (jurisdiction == 1) {
+			site.server.SetExtraParameter("jurisdiction", L"eu");
+			site.server.SetHost(L"eu.r2.cloudflarestorage.com", site.server.GetPort());
+		}
+		else if (jurisdiction == 2) {
+			site.server.SetExtraParameter("jurisdiction", L"fedramp");
+			site.server.SetHost(L"fedramp.r2.cloudflarestorage.com", site.server.GetPort());
+		}
+	}
+
 	if (site.credentials.encrypted_) {
 		if (!pw.empty()) {
 			site.credentials.encrypted_ = fz::public_key();
@@ -782,13 +896,10 @@ void GeneralSiteControls::SetProtocol(ServerProtocol protocol)
 	if (protocol == UNKNOWN) {
 		protocol = FTP;
 	}
-	wxChoice* pProtocol = XRCCTRL(parent_, "ID_PROTOCOL", wxChoice);
-	wxChoice* pEncryption = XRCCTRL(parent_, "ID_ENCRYPTION", wxChoice);
-	wxStaticText* pEncryptionDesc = XRCCTRL(parent_, "ID_ENCRYPTION_DESC", wxStaticText);
 
 	auto const entry = findGroup(protocol);
 	if (entry.first != protocolGroups().cend()) {
-		pEncryption->Clear();
+		encryption_->Clear();
 		for (auto const& prot : entry.first->protocols) {
 			std::wstring name = prot.second;
 			if (!CServer::ProtocolHasFeature(prot.first, ProtocolFeature::Security)) {
@@ -796,44 +907,44 @@ void GeneralSiteControls::SetProtocol(ServerProtocol protocol)
 				name += 0x26a0; // Unicode's warning emoji
 				name += 0xfe0f; // Variant selector, makes it colorful
 			}
-			pEncryption->AppendString(name);
+			encryption_->AppendString(name);
 		}
-		pEncryption->Show();
-		pEncryptionDesc->Show();
-		pEncryption->SetSelection(entry.second - entry.first->protocols.cbegin());
+		encryption_->Show();
+		encryption_desc_->Show();
+		encryption_->SetSelection(entry.second - entry.first->protocols.cbegin());
 	}
 	else {
-		pEncryption->Hide();
-		pEncryptionDesc->Hide();
+		encryption_->Hide();
+		encryption_desc_->Hide();
 	}
 
 	auto const protoIt = mainProtocolListIndex_.find(protocol);
 	if (protoIt != mainProtocolListIndex_.cend()) {
-		pProtocol->SetSelection(protoIt->second);
+		protocol_choice_->SetSelection(protoIt->second);
 	}
 	else if (protocol != ServerProtocol::UNKNOWN) {
 		auto const entry = findGroup(protocol);
 		if (entry.first != protocolGroups().cend()) {
-			mainProtocolListIndex_[protocol] = pProtocol->Append(entry.first->name);
+			mainProtocolListIndex_[protocol] = protocol_choice_->Append(entry.first->name);
 			for (auto const& sub : entry.first->protocols) {
 				mainProtocolListIndex_[sub.first] = mainProtocolListIndex_[protocol];
 			}
 		}
 		else {
-			mainProtocolListIndex_[protocol] = pProtocol->Append(CServer::GetProtocolName(protocol));
+			mainProtocolListIndex_[protocol] = protocol_choice_->Append(CServer::GetProtocolName(protocol));
 		}
 
-		pProtocol->SetSelection(mainProtocolListIndex_[protocol]);
+		protocol_choice_->SetSelection(mainProtocolListIndex_[protocol]);
 	}
 	else {
-		pProtocol->SetSelection(mainProtocolListIndex_[FTP]);
+		protocol_choice_->SetSelection(mainProtocolListIndex_[FTP]);
 	}
 	UpdateHostFromDefaults(GetProtocol());
 }
 
 ServerProtocol GeneralSiteControls::GetProtocol() const
 {
-	int const sel = xrc_call(parent_, "ID_PROTOCOL", &wxChoice::GetSelection);
+	int const sel = protocol_choice_->GetSelection();
 
 	ServerProtocol protocol = UNKNOWN;
 	for (auto const it : mainProtocolListIndex_) {
@@ -845,7 +956,7 @@ ServerProtocol GeneralSiteControls::GetProtocol() const
 
 	auto const group = findGroup(protocol);
 	if (group.first != protocolGroups().cend()) {
-		int encSel = xrc_call(parent_, "ID_ENCRYPTION", &wxChoice::GetSelection);
+		int encSel = encryption_->GetSelection();
 		if (encSel < 0 || encSel >= static_cast<int>(group.first->protocols.size())) {
 			encSel = 0;
 		}
@@ -873,8 +984,8 @@ void GeneralSiteControls::SetControlState()
 {
 	xrc_call(parent_, "ID_HOST", &wxWindow::Enable, !predefined_);
 	xrc_call(parent_, "ID_PORT", &wxWindow::Enable, !predefined_);
-	xrc_call(parent_, "ID_PROTOCOL", &wxWindow::Enable, !predefined_);
-	xrc_call(parent_, "ID_ENCRYPTION", &wxWindow::Enable, !predefined_);
+	protocol_choice_->Enable(!predefined_);
+	encryption_->Enable(!predefined_);
 	xrc_call(parent_, "ID_LOGONTYPE", &wxWindow::Enable, !predefined_);
 
 	xrc_call(parent_, "ID_USER", &wxTextCtrl::Enable, !predefined_ && logonType_ != LogonType::anonymous);
@@ -890,6 +1001,35 @@ void GeneralSiteControls::SetControlState()
 	}
 }
 
+bool GeneralSiteControls::ValidateGoogleAccountKeyFile(std::wstring_view filename)
+{
+	return true;
+}
+
+void GeneralSiteControls::UpdateWidth(int width)
+{
+	wxSize const descSize = encryption_desc_->GetSize();
+	wxSize const encSize = encryption_->GetSize();
+
+	int dataWidth = std::max(encSize.GetWidth(), protocol_choice_->GetSize().GetWidth());
+
+	auto generalSizer = static_cast<wxGridBagSizer*>(protocol_choice_->GetContainingSizer());
+	width = std::max(width, static_cast<int>(descSize.GetWidth() * 2 + dataWidth + generalSizer->GetHGap() * 3));
+
+	wxSize page_min_size = parent_.GetSizer()->GetMinSize();
+	if (page_min_size.x < width) {
+		page_min_size.x = width;
+		parent_.GetSizer()->SetMinSize(page_min_size);
+	}
+
+	// Set min height of general page sizer
+	generalSizer->SetMinSize(generalSizer->GetMinSize());
+
+	// Set min height of encryption row
+	auto encSizer = encryption_->GetContainingSizer();
+	encSizer->GetItem(encSizer->GetItemCount() - 1)->SetMinSize(0, std::max(descSize.GetHeight(), encSize.GetHeight()));
+}
+
 AdvancedSiteControls::AdvancedSiteControls(wxWindow & parent, DialogLayout const& lay, wxFlexGridSizer & sizer)
     : SiteControls(parent)
 {
@@ -908,6 +1048,7 @@ AdvancedSiteControls::AdvancedSiteControls(wxWindow & parent, DialogLayout const
 	for (int i = 0; i < SERVERTYPE_MAX; ++i) {
 		types->Append(CServer::GetNameFromServerType(static_cast<ServerType>(i)));
 	}
+	types->Select(0);
 
 	sizer.AddSpacer(0);
 	sizer.Add(new wxCheckBox(&parent, XRCID("ID_BYPASSPROXY"), _("B&ypass proxy")));
@@ -1260,13 +1401,14 @@ S3SiteControls::S3SiteControls(wxWindow & parent, DialogLayout const& lay, wxFle
 		sizer.AddGrowableCol(0);
 	}
 
-	sizer.Add(new wxStaticText(&parent, nullID, _("Options:")));
 	auto * options_row = lay.createFlex(2);
 	options_row->AddGrowableCol(1);
-	sizer.Add(options_row, 0, wxLEFT|wxGROW, lay.indent);
-	options_row->Add(new wxStaticText(&parent, nullID, _("Re&gion:")), lay.valign);
+	sizer.Add(options_row, lay.grow);
+	options_row->Add(new wxStaticText(&parent, nullID, _("Initial re&gion:")), lay.valign);
 	options_row->Add(new wxTextCtrlEx(&parent, XRCID("ID_S3_REGION")), lay.valigng);
 
+	options_row->Add(new wxCheckBox(&parent, XRCID("ID_S3_ACCELERATE"), _("Use a&ccelerated endpoint")));
+
 	sizer.Add(new wxStaticLine(&parent), lay.grow);
 	sizer.Add(new wxStaticText(&parent, nullID, _("Server Side Encryption:")));
 
@@ -1286,6 +1428,7 @@ S3SiteControls::S3SiteControls(wxWindow & parent, DialogLayout const& lay, wxFle
 	auto * choice = new wxChoice(&parent, XRCID("ID_S3_KMSKEY"));
 	choice->Append(_("Default (AWS/S3)"));
 	choice->Append(_("Custom KMS ARN"));
+	choice->SetSelection(0);
 	key_row->Add(choice, lay.valigng);
 	key_row->Add(new wxStaticText(&parent, nullID, _("C&ustom KMS ARN:")), lay.valign);
 	key_row->Add(new wxTextCtrlEx(&parent, XRCID("ID_S3_CUSTOM_KMS")), lay.valigng);
@@ -1298,16 +1441,36 @@ S3SiteControls::S3SiteControls(wxWindow & parent, DialogLayout const& lay, wxFle
 	key_row->Add(new wxStaticText(&parent, nullID, _("Customer Ke&y:")), lay.valign);
 	key_row->Add(new wxTextCtrlEx(&parent, XRCID("ID_S3_CUSTOMER_KEY")), lay.valigng);
 
-	sizer.Add(new wxStaticLine(&parent), lay.grow);
-	sizer.Add(new wxStaticText(&parent, nullID, _("Security Token Service:")));
+	sizer.Add(new wxStaticLine(&parent, nullID), lay.grow);
+
+	sts_sizer_ = lay.createFlex(1);
+	sts_sizer_->AddGrowableCol(0);
+	sizer.Add(sts_sizer_, lay.grow);
+	sts_sizer_->Add(new wxStaticText(&parent, nullID, _("Security Token Service:")));
 
 	auto * sts_row = lay.createFlex(2);
 	sts_row->AddGrowableCol(1);
-	sizer.Add(sts_row, 0, wxLEFT|wxGROW, lay.indent);
+	sts_sizer_->Add(sts_row, 0, wxLEFT | wxGROW, lay.indent);
 	sts_row->Add(new wxStaticText(&parent, nullID, _("Ro&le ARN:")), lay.valign);
 	sts_row->Add(new wxTextCtrlEx(&parent, XRCID("ID_S3_ROLE_ARN")), lay.valigng);
 	sts_row->Add(new wxStaticText(&parent, nullID, _("MFA D&evice Serial:")), lay.valign);
 	sts_row->Add(new wxTextCtrlEx(&parent, XRCID("ID_S3_MFA_SERIAL")), lay.valigng);
+	sts_sizer_->Show(false);
+
+	sso_sizer_ = lay.createFlex(1);
+	sso_sizer_->AddGrowableCol(0);
+	sizer.Add(sso_sizer_, lay.grow);
+	sso_sizer_->Add(new wxStaticText(&parent, nullID, _("S3 via IAM Identity Center (formerly SSO):")));
+
+	auto * sso_row = lay.createFlex(2);
+	sso_row->AddGrowableCol(1);
+	sso_sizer_->Add(sso_row, 0, wxLEFT | wxGROW, lay.indent);
+	sso_row->Add(new wxStaticText(&parent, nullID, _("Regio&n:")), lay.valign);
+	sso_row->Add(new wxTextCtrlEx(&parent, XRCID("ID_S3_SSO_REGION")), lay.valigng);
+	sso_row->Add(new wxStaticText(&parent, nullID, _("Ro&le name:")), lay.valign);
+	sso_row->Add(new wxTextCtrlEx(&parent, XRCID("ID_S3_SSO_ROLE")), lay.valigng);
+	sso_row->Add(new wxStaticText(&parent, nullID, _("Start &URL:")), lay.valign);
+	sso_row->Add(new wxTextCtrlEx(&parent, XRCID("ID_S3_SSO_URL")), lay.valigng);
 
 	auto l = [this](wxEvent const&) { SetControlState(); };
 	none->Bind(wxEVT_RADIOBUTTON, l);
@@ -1344,10 +1507,16 @@ void S3SiteControls::SetSite(Site const& site)
 	xrc_call(parent_, "ID_S3_AWSKMS", &wxWindow::Enable, !predefined_);
 	xrc_call(parent_, "ID_S3_CUSTOMER_ENCRYPTION", &wxWindow::Enable, !predefined_);
 
-	if (site.server.GetProtocol() == S3) {
+	auto& server = site.server;
+	auto const protocol = server.GetProtocol();
+
+	if (protocol == S3 || protocol == S3_SSO) {
 		auto region = site.server.GetExtraParameter("region");
 		xrc_call(parent_, "ID_S3_REGION", &wxTextCtrl::ChangeValue, region);
 
+		auto accelerate = site.server.GetExtraParameter("accelerate");
+		xrc_call(parent_, "ID_S3_ACCELERATE", &wxCheckBox::SetValue, accelerate == L"1");
+
 		xrc_call(parent_, "ID_S3_KMSKEY", &wxChoice::SetSelection, static_cast<int>(s3_sse::KmsKey::DEFAULT));
 		auto sse_algorithm = site.server.GetExtraParameter("ssealgorithm");
 		if (sse_algorithm.empty()) {
@@ -1369,23 +1538,52 @@ void S3SiteControls::SetSite(Site const& site)
 		}
 		else if (sse_algorithm == "customer") {
 			xrc_call(parent_, "ID_S3_CUSTOMER_ENCRYPTION", &wxRadioButton::SetValue, true);
-			auto customer_key = site.server.GetExtraParameter("ssecustomerkey");
+			auto customer_key = server.GetExtraParameter("ssecustomerkey");
 			xrc_call(parent_, "ID_S3_CUSTOMER_KEY", &wxTextCtrl::ChangeValue, customer_key);
 		}
 
-		auto value = site.server.GetExtraParameter("stsrolearn");
-		xrc_call(parent_, "ID_S3_ROLE_ARN", &wxTextCtrl::ChangeValue, value);
-		value = site.server.GetExtraParameter("stsmfaserial");
-		xrc_call(parent_, "ID_S3_MFA_SERIAL", &wxTextCtrl::ChangeValue, value);
+		if (protocol == S3) {
+			auto value = site.server.GetExtraParameter("stsrolearn");
+			xrc_call(parent_, "ID_S3_ROLE_ARN", &wxTextCtrl::ChangeValue, value);
+			value = server.GetExtraParameter("stsmfaserial");
+			xrc_call(parent_, "ID_S3_MFA_SERIAL", &wxTextCtrl::ChangeValue, value);
+		}
+		if (protocol == S3_SSO) {
+			auto value = server.GetExtraParameter("ssoregion");
+			xrc_call(parent_, "ID_S3_SSO_REGION", &wxTextCtrl::ChangeValue, value);
+			value = server.GetExtraParameter("ssorole");
+			xrc_call(parent_, "ID_S3_SSO_ROLE", &wxTextCtrl::ChangeValue, value);
+			value = server.GetExtraParameter("ssourl");
+			xrc_call(parent_, "ID_S3_SSO_URL", &wxTextCtrl::ChangeValue, value);
+		}
 	}
 }
 
+void S3SiteControls::SetControlVisibility(ServerProtocol protocol, LogonType)
+{
+	sts_sizer_->Show(protocol == S3);
+	sso_sizer_->Show(protocol == S3_SSO);
+
+	auto * s = sts_sizer_->GetContainingWindow()->GetSizer();
+	s->Layout();
+	s->CalcMin();
+}
+
 bool S3SiteControls::UpdateSite(Site & site, bool silent)
 {
-	CServer & server = site.server;
-	if (server.GetProtocol() == S3) {
+	auto& server = site.server;
+	auto const protocol = server.GetProtocol();
+
+	if (protocol == S3 || protocol == S3_SSO) {
 		server.SetExtraParameter("region", fz::to_wstring(xrc_call(parent_, "ID_S3_REGION", &wxTextCtrl::GetValue)));
 
+		if (xrc_call(parent_, "ID_S3_ACCELERATE", &wxCheckBox::GetValue)) {
+			server.SetExtraParameter("accelerate", L"1");
+		}
+		else {
+			server.ClearExtraParameter("accelerate");
+		}
+
 		if (xrc_call(parent_, "ID_S3_NOENCRYPTION", &wxRadioButton::GetValue)) {
 			server.ClearExtraParameter("ssealgorithm");
 		}
@@ -1442,6 +1640,25 @@ bool S3SiteControls::UpdateSite(Site & site, bool silent)
 				server.SetExtraParameter("stsmfaserial", value);
 			}
 		}
+
+		if (protocol == S3_SSO) {
+			auto role = fz::to_wstring(xrc_call(parent_, "ID_S3_SSO_ROLE", &wxTextCtrl::GetValue));
+			auto url = fz::to_wstring(xrc_call(parent_, "ID_S3_SSO_URL", &wxTextCtrl::GetValue));
+			if (role.empty() || url.empty()) {
+				if (!silent) {
+					xrc_call(parent_, "ID_S3_SSO_ROLE", &wxWindow::SetFocus);
+					wxMessageBoxEx(_("You have to enter both role name and start URL."), _("Site Manager - Invalid data"), wxICON_EXCLAMATION, wxGetTopLevelParent(&parent_));
+				}
+				return false;
+			}
+			server.SetExtraParameter("ssorole", role);
+			server.SetExtraParameter("ssourl", url);
+
+			value = fz::to_wstring(xrc_call(parent_, "ID_S3_SSO_REGION", &wxTextCtrl::GetValue));
+			if (!value.empty()) {
+				server.SetExtraParameter("ssoregion", value);
+			}
+		}
 	}
 
 	return true;
