@@ -363,7 +363,7 @@ bool CSiteManagerDialog::Create(wxWindow* parent, std::vector<_connected_site>*
 	auto right = new wxBoxSizer(wxVERTICAL);
 	sides->Add(right, 1, wxLEFT|wxGROW, lay.gap);
 
-	m_pNotebook_Site = new CSiteManagerSite(*this, options_);
+	m_pNotebook_Site = new CSiteManagerSite(options_);
 	if (!m_pNotebook_Site->Load(this)) {
 		return false;
 	}
@@ -744,7 +744,7 @@ bool CSiteManagerDialog::Load()
 	}
 	CSiteManagerXmlHandler_Tree handler(tree_, treeId, lastSelection, false, options_.get_int(OPTION_DEFAULT_KIOSKMODE));
 
-	bool res = CSiteManager::Load(element, handler);
+	bool res = CSiteManager::Load(element, handler, file.GetVersion());
 
 	tree_->SortChildren(treeId);
 	tree_->Expand(treeId);
@@ -842,6 +842,13 @@ bool CSiteManagerDialog::SaveChild(pugi::xml_node element, wxTreeItemId child)
 			bookmarkChild = tree_->GetNextChild(child, cookie);
 		}
 
+		if (data->m_site->server.GetProtocol() == ONEDRIVE) {
+			CSiteManager::UpdateOneDrivePath(data->m_site->m_default_bookmark.m_remoteDir);
+		}
+		else if (data->m_site->server.GetProtocol() == GOOGLE_DRIVE) {
+			CSiteManager::UpdateGoogleDrivePath(data->m_site->m_default_bookmark.m_remoteDir);
+		}
+
 		CSiteManager::Save(node, *data->m_site, CLoginManager::Get(), options_);
 
 		if (data->connected_item != -1) {
@@ -1533,7 +1540,7 @@ bool CSiteManagerDialog::LoadDefaultSites()
 	}
 	CSiteManagerXmlHandler_Tree handler(tree_, m_predefinedSites, lastSelection, true, options_.get_int(OPTION_DEFAULT_KIOSKMODE));
 
-	CSiteManager::Load(element, handler);
+	CSiteManager::Load(element, handler, file.GetVersion());
 
 	return true;
 }
