@@ -9,6 +9,7 @@
 #include "remote_recursive_operation.h"
 #include "listingcomparison.h"
 #include "xrc_helper.h"
+#include "file_utils.h"
 
 #include "../commonui/misc.h"
 
@@ -270,17 +271,40 @@ bool CState::SetLocalDir(std::wstring const& dir, std::wstring *error, bool reme
 		return false;
 	}
 
-	return SetLocalDir(p, error, rememberPreviousSubdir);
+	return DoSetLocalDir(p, error, rememberPreviousSubdir);
 }
 
 bool CState::SetLocalDir(CLocalPath const& dir, std::wstring *error, bool rememberPreviousSubdir)
+{
+	CLocalPath p(dir);
+	return DoSetLocalDir(p, error, rememberPreviousSubdir);
+}
+
+bool CState::DoSetLocalDir(CLocalPath & dir, std::wstring *error, bool rememberPreviousSubdir)
 {
 	if (m_changeDirFlags.syncbrowse) {
 		wxMessageBoxEx(_("Cannot change directory, there already is a synchronized browsing operation in progress."), _("Synchronized browsing"));
 		return false;
 	}
 
-	if (!dir.Exists(error)) {
+	size_t links{};
+	while (++links < 5) {
+		bool is_link{};
+		if (!dir.Exists(error, &is_link)) {
+			return false;
+		}
+		if (!is_link) {
+			break;
+		}
+
+		if (!dir.ChangePath(fz::to_wstring(fz::local_filesys::get_link_target(fz::to_native(dir.GetPath()))))) {
+			return false;
+		}
+	}
+	if (links >= 5) {
+		if (error) {
+			*error = _("Too many symbolic links");
+		}
 		return false;
 	}
 
@@ -855,7 +879,7 @@ void DoHandleDroppedFiles(CState & state, CMainFrame & mainFrame, T const& files
 	op.pTo = to;
 	op.wFunc = copy ? FO_COPY : FO_MOVE;
 	op.hwnd = (HWND)mainFrame.GetHandle();
-	SHFileOperation(&op);
+	CallSHFileOperation(&mainFrame, op);
 
 	delete[] to;
 	delete[] from;
@@ -1079,17 +1103,34 @@ void CState::ListingFailed(int)
 
 	if (m_changeDirFlags.syncbrowse && !m_sync_browse.target_path.empty()) {
 		wxDialogEx dlg;
-		if (!dlg.Load(0, _T("ID_SYNCBROWSE_NONEXISTING"))) {
+		if (!dlg.Create(nullptr, nullID, _("Synchronized browsing"))) {
 			SetSyncBrowse(false);
 			return;
 		}
 
-		xrc_call(dlg, "ID_SYNCBROWSE_NONEXISTING_LABEL", &wxStaticText::SetLabel, wxString::Format(_("The remote directory '%s' does not exist."), m_sync_browse.target_path.GetPath()));
-		xrc_call(dlg, "ID_SYNCBROWSE_CREATE", &wxRadioButton::SetLabel, _("Create &missing remote directory and enter it"));
-		xrc_call(dlg, "ID_SYNCBROWSE_DISABLE", &wxRadioButton::SetLabel, _("&Disable synchronized browsing and continue changing the local directory"));
+		auto & lay = dlg.layout();
+		auto top = lay.createMain(&dlg, 2);
+
+		top->Add(new wxStaticBitmap(&dlg, nullID, wxArtProvider::GetBitmap(wxART_QUESTION, wxART_OTHER, wxSize(32, 32))), 0, wxALL, lay.dlgUnits(2));
+
+		auto main = lay.createFlex(1);
+		top->Add(main);
+
+		main->Add(new wxStaticText(&dlg, nullID, wxString::Format(_("The remote directory '%s' does not exist."), LabelEscape(m_sync_browse.target_path.GetPath()))));
+
+		auto create = new wxRadioButton(&dlg, nullID, _("Create &missing remote directory and enter it"), wxDefaultPosition, wxDefaultSize, wxRB_GROUP);
+		main->Add(create);
+		create->SetFocus();
+		auto disable = new wxRadioButton(&dlg, nullID, _("&Disable synchronized browsing and continue changing the local directory"));
+		main->Add(disable);
+
+		lay.createButtonSizerButtons(&dlg, main, false);
+
 		dlg.GetSizer()->Fit(&dlg);
+		dlg.GetSizer()->SetSizeHints(&dlg);
+
 		if (dlg.ShowModal() == wxID_OK) {
-			if (xrc_call(dlg, "ID_SYNCBROWSE_CREATE", &wxRadioButton::GetValue)) {
+			if (create->GetValue()) {
 				transfer_flags const flags = GetMkdirFlags(m_site.server, m_mainFrame.GetOptions(), m_sync_browse.target_path);
 				m_pCommandQueue->ProcessCommand(new CMkdirCommand(m_sync_browse.target_path, flags));
 				m_pCommandQueue->ProcessCommand(new CListCommand(m_sync_browse.target_path));
@@ -1157,21 +1198,37 @@ bool CState::ChangeRemoteDir(CServerPath const& path, std::wstring const& subdir
 				SetSyncBrowse(false);
 			}
 			else if (!local_path.Exists(&error)) {
-				wxString msg = error + _T("\n") + _("Disable synchronized browsing and continue changing the remote directory?");
-
 				wxDialogEx dlg;
-				if (!dlg.Load(0, _T("ID_SYNCBROWSE_NONEXISTING"))) {
+				if (!dlg.Create(nullptr, nullID, _("Synchronized browsing"))) {
 					return false;
 				}
-				xrc_call(dlg, "ID_SYNCBROWSE_NONEXISTING_LABEL", &wxStaticText::SetLabel, wxString::Format(_("The local directory '%s' does not exist."), local_path.GetPath()));
-				xrc_call(dlg, "ID_SYNCBROWSE_CREATE", &wxRadioButton::SetLabel, _("Create &missing local directory and enter it"));
-				xrc_call(dlg, "ID_SYNCBROWSE_DISABLE", &wxRadioButton::SetLabel, _("&Disable synchronized browsing and continue changing the remote directory"));
+
+				auto & lay = dlg.layout();
+				auto top = lay.createMain(&dlg, 2);
+
+				top->Add(new wxStaticBitmap(&dlg, nullID, wxArtProvider::GetBitmap(wxART_QUESTION, wxART_OTHER, wxSize(32, 32))), 0, wxALL, lay.dlgUnits(2));
+
+				auto main = lay.createFlex(1);
+				top->Add(main);
+
+				main->Add(new wxStaticText(&dlg, nullID, wxString::Format(_("The local directory '%s' does not exist."), LabelEscape(local_path.GetPath()))));
+
+				auto create = new wxRadioButton(&dlg, nullID, _("Create &missing local directory and enter it"), wxDefaultPosition, wxDefaultSize, wxRB_GROUP);
+				main->Add(create);
+				create->SetFocus();
+				auto disable = new wxRadioButton(&dlg, nullID, _("&Disable synchronized browsing and continue changing the remote directory"));
+				main->Add(disable);
+
+				lay.createButtonSizerButtons(&dlg, main, false);
+
 				dlg.GetSizer()->Fit(&dlg);
+				dlg.GetSizer()->SetSizeHints(&dlg);
+
 				if (dlg.ShowModal() != wxID_OK) {
 					return false;
 				}
 
-				if (xrc_call(dlg, "ID_SYNCBROWSE_CREATE", &wxRadioButton::GetValue)) {
+				if (create->GetValue()) {
 					{
 						wxLogNull log;
 						wxMkdir(local_path.GetPath());
